"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class IdiomGame {
    constructor() {
        this.events = {
            start() {
                return;
            },
            answer(answer) {
                console.log(`${answer.userName}：${answer.content}`);
                return void 0;
            },
            end: () => {
                this.stopTimer();
                return void 0;
            }
        };
        this.$score = {};
        this.$stack = [];
        this.$status = "idle";
    }
    get status() {
        return this.$status;
    }
    lastIdiom() {
        return this.$stack[this.$stack.length - 1];
    }
    getScore(id) {
        if (this.$score[id] && typeof this.$score[id] === "number")
            return this.$score[id].score;
        return null;
    }
    getAllScore() {
        return this.$score;
    }
    addScore(id, name, score = 1) {
        if (!this.$score[id])
            this.$score[id] = { name, score };
        else
            this.$score[id].score += score;
    }
    push(idiom) {
        this.$stack.push(idiom);
    }
    startTimer(timeout = 30000) {
        this.$timer = setTimeout(() => __awaiter(this, void 0, void 0, function* () {
            if (this.$status === "start")
                yield this.end();
        }), timeout);
        this.timerStartAt = new Date().getTime();
    }
    stopTimer() {
        clearTimeout(this.$timer);
    }
    resetTimer() {
        if (this.$timer) {
            this.$timer.refresh();
            this.timerStartAt = new Date().getTime();
        }
    }
    on(event, listener) {
        this.events[event] = listener;
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.$status !== "idle")
                return;
            this.$status = "starting";
            this.events.start();
            this.$status = "start";
        });
    }
    pause() {
        if (this.$status !== "start")
            return;
        this.$status = "pause";
    }
    continue() {
        if (this.$status !== "pause")
            return;
        this.$status = "start";
    }
    answer(answer) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.$status !== "start")
                return;
            return this.events.answer(answer);
        });
    }
    end(...args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.$status !== "start" && this.$status !== "pause")
                return;
            this.$status = "ending";
            this.events.end(...args);
            this.$score = {};
            this.$stack = [];
            this.$status = "idle";
        });
    }
}
exports.default = IdiomGame;
