"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const NeteaseCloudMusicApi_1 = __importDefault(require("NeteaseCloudMusicApi"));
const wechaty_1 = require("wechaty");
const Interceptor_1 = __importDefault(require("../../Interceptor"));
// 这个方法用于解析搜索参数
function getSearchArgs(text) {
    if (!text || text.length === 0)
        return null;
    let match;
    const singerRegExp = new RegExp("我想听(.*)的歌");
    match = singerRegExp.exec(text);
    if (match && match.length > 0)
        return {
            type: 100 /* artist */,
            keywords: match[1]
        };
    const singerAndSingleRegExp = new RegExp("我想听(.*)唱?的(.*)");
    match = singerAndSingleRegExp.exec(text);
    if (match && match.length > 0)
        return {
            type: 1 /* single */,
            keywords: `${match[1]} ${match[2]}`
        };
    const singleRegExp = new RegExp("我想听(.*)");
    match = singleRegExp.exec(text);
    if (match && match.length > 0)
        return {
            type: 1 /* single */,
            keywords: match[1]
        };
    return null;
}
const neteaseMusicInterceptor = new Interceptor_1.default("netease-cloud-music", context => {
    context.template.add("netease-cloud-music.search.failed", "很抱歉，搜索 {search} 时失败了，请你等会再试试吧。");
    context.template.add("netease-cloud-music.search.success", "二师兄给你找到了一首歌~<br/>{title} - {artists}<br/>{url}");
})
    .title("网易云音乐")
    .alias("搜音乐")
    .usage("二师兄，我想听……")
    .check((context, message) => {
    const searchArgs = getSearchArgs(message.text());
    if (/^二师兄/.test(message.text()) && searchArgs) {
        return { searchArgs };
    }
})
    .handler((context, message, checkerArgs) => __awaiter(void 0, void 0, void 0, function* () {
    const { searchArgs } = checkerArgs;
    // 调用搜索API
    const searchResult = (yield NeteaseCloudMusicApi_1.default.search(Object.assign({}, searchArgs)));
    if (searchResult.status === 200 && searchResult.body.result.songs.length > 0) {
        const resultSong = searchResult.body.result.songs[0];
        // 调用歌曲详情API
        const resultSongDetailResponse = (yield NeteaseCloudMusicApi_1.default.song_detail({
            ids: `${resultSong.id}`
        }));
        if (resultSongDetailResponse.status === 200 && resultSongDetailResponse.body.songs.length > 0) {
            const resultSongDetail = resultSongDetailResponse.body.songs[0];
            // 获取展示需要的信息
            const thumbnail = resultSongDetail.al.picUrl;
            const url = `https://music.163.com/#/song?id=${resultSongDetail.id}`;
            const title = resultSongDetail.name;
            const artists = resultSongDetail.ar.map(a => a.name).join("/");
            // 创建链接卡片
            const urlLink = new wechaty_1.UrlLink({
                title,
                description: artists,
                url,
                thumbnailUrl: thumbnail
            });
            console.log(urlLink);
            try {
                yield message.say(urlLink);
            }
            catch (e) {
                // 发送失败时，回落到文本消息
                // 注：不明白为什么wechaty-puppet-wechat不能发UrlLink……看说明应该可以发才对，但输出是不支持……
                // 2021-08-31：后来看了看文档，web协议不支持，那没事了
                console.warn("Send UrlLink failed, fallback to text message.");
                console.warn(e);
                yield message.say(context.template.use("netease-cloud-music.search.success", { title, artists, url }));
            }
            return "";
        }
    }
    return context.template.use("netease-cloud-music.search.failed", {
        search: searchArgs.keywords
    });
}));
exports.default = neteaseMusicInterceptor;
