"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class PuzzleGame {
    constructor() {
        this.events = {
            start() {
                return;
            },
            answer(answer) {
                console.log(`${answer.userName}：${answer.content}`);
                return void 0;
            },
            end: () => {
                this.stopTimer();
                return void 0;
            }
        };
        this.$status = "idle";
    }
    start(timeout = 30000) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.$status !== "idle")
                return;
            this.$status = "starting";
            yield this.events.start();
            this.startTimer(timeout);
            this.$status = "start";
        });
    }
    answer(answer) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.$status !== "start")
                return;
            return this.events.answer(answer);
        });
    }
    end(...args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.$status !== "start" && this.$status !== "pause")
                return;
            this.$status = "ending";
            this.stopTimer();
            yield this.events.end(...args);
            this.$status = "idle";
        });
    }
    startTimer(timeout = 30000) {
        if (!this.$timer) {
            this.$timer = setTimeout(() => {
                void this.end();
            }, timeout);
        }
    }
    stopTimer() {
        if (this.$timer) {
            clearTimeout(this.$timer);
            this.$timer = null;
        }
    }
    get status() {
        return this.$status;
    }
    on(event, listener) {
        this.events[event] = listener;
    }
    set data(data) {
        this.$data = data;
    }
    get data() {
        return this.$data;
    }
}
exports.default = PuzzleGame;
