"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const PuzzleGame_1 = __importDefault(require("./PuzzleGame"));
const tianapi_1 = __importDefault(require("./tianapi"));
const Interceptor_1 = __importDefault(require("../../Interceptor"));
const gameSpace = {};
const defaultWordPuzzleConfig = () => {
    return {
        limit: 5
    };
};
const wordPuzzleInterceptor = new Interceptor_1.default("word-puzzle", (context) => {
    context.template.add("word-puzzle.game.start", [
        "来啦，猜字谜：<br/>{puzzle}",
        "要猜字谜吗？那么来试试吧：<br/>{puzzle}"
    ]);
    context.template.add("word-puzzle.game.playing", "当前正在进行一个猜字谜游戏，赶紧回答吧。");
    context.template.add("word-puzzle.game.win", [
        "{name}答对啦！<br/>谜底是：{answer}<br/>{reason}"
    ]);
    context.template.add("word-puzzle.game.lose", [
        "没有人回答出来吗？那么二师兄公布答案了。<br/>谜底是：{answer}<br/>{reason}"
    ]);
})
    .title("猜字谜")
    .alias("字谜")
    .attribute("limit", (context, { id }) => context.callLimiter.check(`room_${id}`, "word-puzzle"), "今日游玩次数")
    .usage("来和二师兄猜字谜吧！")
    .check((context, message) => {
    const contact = message.talker();
    const room = message.room();
    const spaceId = room ?
        `room_${room.id}` :
        `contact_${contact.id}`;
    const args = { contact, room, spaceId };
    if (/^二师兄/.test(message.text()) && /猜字谜/.test(message.text())) {
        return Object.assign(Object.assign({}, args), { action: "start" });
    }
    else if (gameSpace[spaceId] && gameSpace[spaceId].status === "start") {
        return Object.assign(Object.assign({}, args), { action: "answer", content: message.text() });
    }
})
    .handler((context, message, checkerArgs) => __awaiter(void 0, void 0, void 0, function* () {
    const { spaceId, action, content } = checkerArgs;
    if (action === "start") {
        const config = Object.assign(Object.assign({}, defaultWordPuzzleConfig()), context.getMethodConfig("word_puzzle"));
        if (!gameSpace[spaceId] || gameSpace[spaceId].status === "idle") {
            if (config.limit && (yield context.callLimiter.check(spaceId, "word-puzzle")) >= config.limit)
                return context.template.use("word-puzzle.game.limit");
            gameSpace[spaceId] = new PuzzleGame_1.default();
            gameSpace[spaceId].on("start", () => __awaiter(void 0, void 0, void 0, function* () {
                try {
                    const puzzle = yield tianapi_1.default();
                    yield context.callLimiter.record(spaceId, "word-puzzle");
                    gameSpace[spaceId].data = puzzle;
                    yield message.say(context.template.use("word-puzzle.game.start", {
                        puzzle: puzzle.content
                    }));
                }
                catch (e) {
                    throw e;
                }
            }));
            gameSpace[spaceId].on("answer", (answer) => __awaiter(void 0, void 0, void 0, function* () {
                if (answer.content === gameSpace[spaceId].data.answer) {
                    yield gameSpace[spaceId].end(answer.userName);
                    return true;
                }
            }));
            gameSpace[spaceId].on("end", (userName) => __awaiter(void 0, void 0, void 0, function* () {
                if (userName)
                    yield message.say(context.template.use("word-puzzle.game.win", {
                        name: userName,
                        answer: gameSpace[spaceId].data.answer,
                        reason: gameSpace[spaceId].data.reason
                    }));
                else
                    yield message.say(context.template.use("word-puzzle.game.lose", {
                        answer: gameSpace[spaceId].data.answer,
                        reason: gameSpace[spaceId].data.reason
                    }));
            }));
            yield gameSpace[spaceId].start();
        }
        else {
            return context.template.use("word-puzzle.game.playing");
        }
    }
    else if (action === "answer") {
        const result = yield gameSpace[spaceId].answer({
            content,
            userId: message.talker().id,
            userName: message.talker().name()
        });
        if (result)
            return "";
    }
}));
exports.default = wordPuzzleInterceptor;
