"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.place = exports.getAPIKey = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const bot_1 = require("../../bot");
const axios_1 = __importDefault(require("axios"));
const axios = axios_1.default.create();
/**
 * 获取指定API的密钥，如果密钥不存在会抛出“ERR_CANNOT_GET_KEY”异常
 * @param {string} name 要获取密钥的API的名称
 * @returns {string} 目标API密钥
 */
function getAPIKey(name) {
    try {
        const data = fs_1.default.readFileSync(path_1.default.join(bot_1.__data_dir, "./api/caiyunapi.json"), "utf-8");
        const dataParsed = JSON.parse(data);
        if (!typeof dataParsed.key)
            throw new Error("Property \"key\" not defined");
        if (!typeof dataParsed.key[name])
            throw new Error("Property \"key." + name + "\" not defined");
        if (typeof dataParsed.key[name] !== "string")
            throw new Error("Property \"key." + name + "\" is not string");
        return dataParsed.key[name];
    }
    catch (e) {
        console.error("获取 CaiyunAPI[" + name + "] 的密钥时出错");
        console.error(e);
        throw new Error("ERR_CANNOT_GET_KEY");
    }
}
exports.getAPIKey = getAPIKey;
function place(query) {
    return new Promise((resolve, reject) => {
        axios({
            url: "https://api.caiyunapp.com/v2/place",
            params: {
                token: "96Ly7wgKGq6FhllM",
                count: 1,
                lang: "zh_CN",
                query,
                random: Math.random(),
                _: new Date().getTime()
            }
        }).then((value) => {
            const data = value.data;
            if (data.status === "ok" && data.places && data.places[0])
                resolve({
                    name: data.places[0].name,
                    address: data.places[0].formatted_address,
                    location: data.places[0].location
                });
            else
                reject(value.data.message || "ERR_UNKNOWN");
        }).catch(reject);
    });
}
exports.place = place;
