"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAPIKey = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const bot_1 = require("../../bot");
/**
 * 获取指定API的密钥，如果密钥不存在会抛出“ERR_CANNOT_GET_KEY”异常
 * @param {string} name 要获取密钥的API的名称
 * @returns {string} 目标API密钥
 */
function getAPIKey(name) {
    try {
        const data = fs_1.default.readFileSync(path_1.default.join(bot_1.__data_dir, "./api/juheapi.json"), "utf-8");
        const dataParsed = JSON.parse(data);
        if (!dataParsed.key)
            throw new Error("Property \"key\" not defined");
        if (!dataParsed.key[name])
            throw new Error("Property \"key." + name + "\" not defined");
        if (typeof dataParsed.key[name] !== "string")
            throw new Error("Property \"key." + name + "\" is not string");
        return dataParsed.key[name];
    }
    catch (e) {
        console.error("获取 JuheAPI[" + name + "] 的密钥时出错");
        console.error(e);
        throw new Error("ERR_CANNOT_GET_KEY");
    }
}
exports.getAPIKey = getAPIKey;
