"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class CallLimiter {
    /**
     * This constructor has a async create table call.
     * So, please, create this class as early as possible.
     * @param databaseTemplate
     */
    constructor(databaseTemplate) {
        this.databaseTemplate = databaseTemplate;
        const sql = "CREATE TABLE IF NOT EXISTS call_limit (" +
            "identify_id VARCHAR(64) NOT NULL," +
            "method      VARCHAR(64) NOT NULL," +
            "call_time   LONG        NOT NULL" +
            ")";
        this.databaseTemplate.run(sql).catch(console.error);
    }
    /**
     * Check is method reach call limit or not.
     * Default "checkSection" argument is from 00:00:00AM today to 23:59:59 today
     */
    check(identifyId, method, checkSectionMin, checkSectionMax) {
        return __awaiter(this, void 0, void 0, function* () {
            const today = new Date();
            if (!checkSectionMin)
                checkSectionMin = new Date(`${today.toDateString()} 00:00:00`);
            if (!checkSectionMax)
                checkSectionMax = new Date(`${today.toDateString()} 23:59:59`);
            try {
                const sql = "SELECT COUNT(*) FROM call_limit WHERE identify_id = ? AND method = ? AND call_time > ? AND call_time < ?";
                const result = yield this.databaseTemplate.get(sql, identifyId, method, checkSectionMin.getTime(), checkSectionMax.getTime());
                return result["COUNT(*)"];
            }
            catch (e) {
                console.error(e);
                throw new Error("ERR_CHECK_CALL_LIMIT_FAILED");
            }
        });
    }
    record(identifyId, method) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const sql = "INSERT INTO call_limit (identify_id, method, call_time) VALUES (?, ?, ?)";
                const result = yield this.databaseTemplate.run(sql, identifyId, method, new Date().getTime());
                return result.changes;
            }
            catch (e) {
                console.error(e);
                throw new Error("ERR_INSERT_CALL_LIMIT_RECORD_FAILED");
            }
        });
    }
}
exports.default = CallLimiter;
