"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const sqlite3_1 = __importDefault(require("sqlite3"));
const fs_1 = __importDefault(require("fs"));
const Util_1 = require("./Util");
const Database = sqlite3_1.default.verbose().Database;
class SqliteTemplate {
    constructor(file) {
        const path = file.substring(0, file.replace('\\', '/').lastIndexOf('/'));
        if (!fs_1.default.existsSync(file)) {
            console.warn("File not exists: " + file + ", will create it automatically.");
            Util_1.mkdirSync(path);
        }
        this.file = file;
    }
    run(sql, ...params) {
        return new Promise((resolve, reject) => {
            const db = new Database(this.file);
            db.run(sql, params, function (err) {
                const { changes, lastID } = this;
                if (err)
                    reject(err);
                else
                    resolve({ changes, lastID });
                db.close(function (closeErr) {
                    if (closeErr)
                        console.error(closeErr);
                });
            });
        });
    }
    get(sql, ...params) {
        return new Promise((resolve, reject) => {
            const db = new Database(this.file);
            db.get(sql, params, function (err, row) {
                if (err)
                    reject(err);
                else
                    resolve(row);
                db.close(function (closeErr) {
                    if (closeErr)
                        console.error(closeErr);
                });
            });
        });
    }
    all(sql, ...params) {
        return new Promise((resolve, reject) => {
            const db = new Database(this.file);
            db.get(sql, params, function (err, rows) {
                if (err)
                    reject(err);
                else
                    resolve(rows);
                db.close(function (closeErr) {
                    if (closeErr)
                        console.error(closeErr);
                });
            });
        });
    }
    getConnection() {
        return new Database(this.file);
    }
}
exports.default = SqliteTemplate;
