# 开发工具

本章为您介绍开发二师兄机器人时，您可以调用的便于开发的工具。

## 模板

**模板**（**Template**）是定义机器人与用户交流时消息内容的工具。

使用这个工具的意义在于，模块的开发者可以预先定义一套应答方案；而在其他开发者想要使用该模块时，可以根据自己的意愿再定义一套新的应答方案。

此模块推荐的使用标准：

* 为一个键命名时，按照`功能名称.子功能名称.动作描述`的标准命名
* 如果不需要强制覆盖键值，请用`template.add()`

使用方法：

* 如果您是模块开发者

  您需要在您的入口文件中引入`template`，然后调用`template.add()`为您的模块定义一套基础应答方案。

  ```typescript
  import { template } from "../../bot"
  import Interceptor from "./Interceptor";
  template.add("my-method.default", "这是默认的应答方案")
  const interceptor = new Interceptor("my-method")
      .check(message => /^hello/.test(message.text().toLowerCase()))
      .handler(() => template.use("my-method.default"))
  export default interceptor
  ```

  当然您也可以覆盖本项目默认的应答方案，例如您想要覆盖控制台登录信息提示：

  ```typescript
  import { template } from "../../bot"
  template.set("on.login", "行くよ、{name}ちゃん！")
  // 由于启动时，是先载入拦截器再启动机器人，所以登录成功时控制台会显示“行くよ、机器人名字ちゃん！”
  ```

  项目默认的模板值，您可以参考`/src/template.ts`的内容。

* 如果您是模块使用者

  您应该不会因为想要修改一句话的内容，而去修改别人的代码，导致拉取更新的时候带来令人头痛的merge操作。

  这个时候您可以在引入所有拦截器后，引入您的设置文件，用`template.set()`设定应答：

  ```typescript
  // 您的拦截器，例如名为 my-template-interceptor
  import { template } from "../../bot.ts"
  template.set("others-method.default", "修改后的应答内容")
  // /interceptor/entry.ts
  // ...
  import "my-template-interceptor"
  ```
